from logger import Logger


class EventDriver(Logger):
    """
    Abstract factory class for drivers. A driver translates driver-specific
    messages to a common set of events. The minimal set is {point_down,
    point_move, point_up}. A driver implementation should define the methods
    'start' and 'stop', which starts/stops some event loop that triggers the
    'delegate_event' method of a area.
    """
    def __init__(self, event_server):
        self.event_server = event_server

    def start(self):
        """
        Start the event loop.
        """
        raise NotImplementedError

    def stop(self):
        """
        Stop the event loop.
        """
        raise NotImplementedError

    def run(self):
        """
        Execute a single loop iteration. If implemented, this method provides a
        way to run the driver within another event loop.
        """
        raise NotImplementedError
